body {
	font-family: "Arial";
}

div.size_container {
	width: ${renderer.rendering_params.geometry_x}px;
	height: ${renderer.rendering_params.geometry_y}px;
	border: 1px solid #aaa;
	margin-bottom: 10px;
	overflow: hidden;
}

a.slideanchor {
	position: relative;
%if rendered_presentation.has_feature("interactive"):
	top: -100px;
%else:
	top: -10px;
%endif
}

div.slide {
	position: relative;
	height: 100%;
	width: 100%;
	font-size: 48px;
	background: #fff;
}

button {
	border-radius: 5px;
	padding: 10px 30px;
	transition-duration: 0.1s;
	background-color: #eee;
	padding: 10px 30px;
	border: 1px solid #aaa;
	font-size: 18px;
}

button:hover {
	background-color: #ccc;
}

button:disabled {
	background-color: #efefef;
}

button:active {
	box-shadow: 0 1px 5px rgba(0, 0, 0, 0.5);
}

@keyframes rotation {
	0% {
		transform: rotate(0deg) translateY(-1px);
	}

	100% {
		transform: rotate(359deg) translateY(-1px);
	}
}

button > span.loader {
	display: none;
}

button.loading > span.loader {
	display: inline-block;
	animation: rotation 1.5s infinite linear;
	margin-left: 5px;
	margin-right: 5px;
}

input {
	width: 40%;
}

input, textarea {
	border-radius: 5px;
	border: 1px solid #aaa;
	padding: 10px;
	font-family: "Arial";
	font-size: 18px;
}

span.symbol {
	font-family: math;
}

pre {
	font-size: 75%;
	margin-top: 0.5em;
	margin-bottom: 0.5em;
}

span.tt {
	font-family: monospace;
}

ul, ol, p {
	margin-block-start: 0.5em;
	margin-block-end: 0.5em;
}

ul, ol {
	margin-block-start: 0;
	margin-block-end: 0;
}

pre.terminal {
	background: #111;
	color: #eee;
	border: 1px gray dotted;
	margin-left: 3%;
	margin-right: 3%;
	padding: 10px;
	overflow-x: auto;
}

pre.terminal .command {
	user-select: all;
}

div.code_highlight {
	background: #eee;
	border: 1px gray dotted;
	margin-left: 3%;
	margin-right: 3%;
	padding: 10px;
	overflow-x: auto;
}

div.fillimg {
	display: flex;
	justify-content: center;
	align-items: center;
	width: 100%;
	height: 100%;
}

div.fillimg img {
	max-width: 100%;
	max-height: 100%;
	display: block;
}


table {
	border-spacing: 0px;
	border-collapse: separate;
}

th, td {
	padding: 5px 15px 5px 15px;
	text-align: left;
	border-bottom: 1px solid #ddd;
}

tr.no-border-bottom th, tr.no-border-bottom td {
	border-bottom: none;
}

tr.fat-border-bottom th, tr.fat-border-bottom td {
	border-bottom: 3px solid #ddd;
}

th.border-right, td.border-right {
	border-right: 1px solid #ddd;
}

/* In presentation mode, make links not clickable */
.fullscreen a {
	pointer-events: none;
}
