<%inherit file="base/slide.html" />
<%block name="slide_classes">title</%block>
<img src="${preuri}template/antonio/bar.svg" class="bar">
<div class="author">${presentation.meta.get("author")}</div>
<div class="title">
	<div class="main">${presentation.meta.get("title")}</div>
	%if presentation.meta.get("subtitle") is not None:
	<div class="sub">${presentation.meta.get("subtitle")}</div>
	%endif
</div>
<div class="date">${presentation.meta.get("date")}</div>
