<%inherit file="template_slide.html" />
<%block name="header">
	<div class="heading">${slide.var("heading", "Acronyms")}</div>
</%block>
<% acronyms = slide.var("acronyms") %>
%if acronyms is None:
Acronyms not available yet.
%else:
<ul>
	%for acronym in acronyms:
	<li>${acronym.acronym}:
	%if acronym.uri is None:
	${acronym.text}
	%else:
	<a href="${acronym.uri}">${acronym.text}</a>
	%endif
	</li>
	%endfor
</ul>
%endif
