<%inherit file="base/slide.html" />
<%block name="slide_classes">quote</%block>
<div class="body">
	<img src="${preuri}template/antonio/quote.svg" class="quotationmark" />
	<div class="flex">
		<div class="quote_text">
		${slide.content() | n}
		</div>
		%if slide.var("author") is not None:
			<div class="quote_author">— ${slide.var("author")}</div>
		%endif
	</div>
</div>
